# Configuration

Basic configuration of PyHDX is done through a `.yaml` configuration file. By default, the configuration
file in the user home directory is used  (`~/.pyhdx/config.yaml`).  
Individual or all config entries can be overwritten by loading (partial) config files.

In `'myconfigfile.yaml`:

```yaml
cluster:
  n_workers: 4
fitting: 
  dtype: float32
  device: cuda
```
Then to load the config: 

```python
from pyhdx.config import cfg
cfg.load_config('myconfigfile.yaml')
cfg.TORCH_DTYPE
>>> torch.float32
```

## Default configuration

The default configuration file is shown below, and can be found in PyHDX source at `pyhdx/config.yaml`.

```yaml
--8<--
./pyhdx/config.yaml
--8<--
```

### Cluster
Settings related to the `dask` cluster for computation of $\Delta G$ values.

- **scheduler_address**: The address for the `dask` scheduler. If scheduler is found at this address,
  a new cluster is created. 
- **n_workers**: Number of workers for the cluster.


### Server
Settings related to the `panel` web application.

- **assets_dir**: Directory for static assets. Uploaded `.pdb` files for visualization are stored here.
- **log_dir**: Directory for log files.

### Fitting
Settings related to $\Delta G$ fitting.

- **dtype**: Data type for fitting. Can be `float32` or `float64`.
- **device**: Device for fitting. Can be `cpu` or `cuda` (GPU), if `cuda` is available.

### Analysis
Settings related to analysis of HDX-MS data.

- **drop_first**: Number of N-terminal residues to consider as completely back-exchanging. 
- **weight_exponent**: Exponent for calculating residue-level RFU values by weighted averaging.

### Plotting
Settings related to output format of plots generated by the web interface. These include widths and 
aspect ratios of figures and plots, as well as the number of columns of plots to place into one figure.